@@ -42,7 +42,7 @@
 arg_pat="$1"
 
 # the version of this script
-bm_version=4.3.0
+bm_version=4.9.0
 
 # CONFIGURABLE VARIABLES ---------------------------------------
 # change as needed to reflect your environment/workloads
@@ -315,6 +315,7 @@ if [[ "$arg_pat" == "reset" ]]; then
   rm -f data_sorted.csv
   rm -f benchmark_data.xlsx
   rm -f benchmark_data.jsonl
+  rm -f benchmark_data.json
   rm -f benchmark_data.schema.json
   rm -f searchset_patterns.txt
   rm -f searchset_patterns_unicode.txt
@@ -362,21 +363,23 @@ fi
 if [ ! -r searchset_patterns_unicode.txt ]; then
   echo "> Preparing benchmark support data..."
   # create an index so benchmark data preparation commands can run faster
-  "$qsv_bin" index "$data"
+  "$qsv_benchmarker_bin" index "$data"
   echo "   data_to_exclude.csv..."
-  "$qsv_bin" sample --seed 42 1000 "$data" -o data_to_exclude.csv
+  "$qsv_benchmarker_bin" sample --seed 42 1000 "$data" -o data_to_exclude.csv
   echo "   data_unsorted.csv..."
-  "$qsv_bin" sort --seed 42 --random --faster "$data" -o data_unsorted.csv
+  "$qsv_benchmarker_bin" sort --seed 42 --random --faster "$data" -o data_unsorted.csv
   echo "   data_sorted.csv..."
-  "$qsv_bin" sort "$data" -o data_sorted.csv
+  "$qsv_benchmarker_bin" sort "$data" -o data_sorted.csv
   echo "   benchmark_data.xlsx..."
   "$qsv_benchmarker_bin" to xlsx benchmark_data.xlsx "$data"
   echo "   benchmark_data.jsonl..."
-  "$qsv_bin" tojsonl "$data" --output benchmark_data.jsonl
+  "$qsv_benchmarker_bin" tojsonl "$data" --output benchmark_data.jsonl
+  echo "   benchmark_data.json..."
+  "$qsv_benchmarker_bin" sqlp --format json "$data" -Q 'select * from _t_1' --infer-len 127000 --rnull-values 'N/A' --output benchmark_data.json
   echo "   benchmark_data.schema.json..."
-  "$qsv_bin" schema "$data" --stdout >benchmark_data.csv.schema.json
+  "$qsv_benchmarker_bin" schema "$data" --stdout >benchmark_data.csv.schema.json
   echo "   benchmark_data.snappy..."
-  "$qsv_bin" snappy compress "$data" --output benchmark_data.snappy
+  "$qsv_benchmarker_bin" snappy compress "$data" --output benchmark_data.snappy
   echo "   searchset_patterns.txt..."
   printf "homeless\npark\nNoise\n" >searchset_patterns.txt
   echo "   searchset_patterns_unicode.txt..."
@@ -446,6 +449,7 @@ run apply_emptyreplace "$qsv_bin" apply emptyreplace \"Bridge Highway Name\" --r
 run apply_op_eudex "$qsv_bin apply operations lower,eudex Agency --comparand Queens --new-column Agency_queens_soundex $data"
 run apply_op_string "$qsv_bin apply operations lower Agency $data"
 run apply_op_similarity "$qsv_bin apply operations lower,simdln Agency --comparand brooklyn --new-column Agency_sim-brooklyn_score $data"
+run apply_op_similarity_batchall "$qsv_bin apply operations lower,simdln Agency --comparand brooklyn --batch 0 --new-column Agency_sim-brooklyn_score $data"
 run behead "$qsv_bin" behead "$data"
 run behead_flexible "$qsv_bin" behead --flexible "$data"
 run cat_columns "$qsv_bin" cat columns "$data" data_unsorted.csv
@@ -461,16 +465,22 @@ run count_width "$qsv_bin" count --width "$data"
 run --index count_width_index "$qsv_bin" count --width "$data"
 run datefmt "$qsv_bin datefmt \"Created Date\" $data"
 run datefmt_multi "$qsv_bin datefmt \"Created Date,Closed Date,Due Date\" $data"
+run datefmt_multi_batchall "$qsv_bin datefmt \"Created Date,Closed Date,Due Date\" --batch 0 $data"
 run datefmt_multi_select "$qsv_bin datefmt '/(?i) date$/' $data"
 run datefmt_formatstr_newcol "$qsv_bin datefmt --formatstr '%V' \"Created Date\" --new-column week_number $data"
 run dedup "$qsv_bin" dedup "$data"
 run dedup_sorted "$qsv_bin" dedup data_sorted.csv
 run diff "$qsv_bin" diff "$data" data_unsorted.csv
 run enum "$qsv_bin" enum "$data"
-run enum_uuid "$qsv_bin" enum --uuid "$data"
+run enum_hash "$qsv_bin" enum --hash 1- "$data"
+run enum_uuid "$qsv_bin" enum --uuid4 "$data"
+run enum_uuid7 "$qsv_bin" enum --uuid7 "$data"
 run enum_constant "$qsv_bin" enum --constant "NYC" "$data"
 run enum_copy "$qsv_bin" enum --copy Agency "$data"
 run excel "$qsv_bin" excel benchmark_data.xlsx
+run excel_j1 "$qsv_bin" excel -j 1 benchmark_data.xlsx
+run excel_trim "$qsv_bin" excel --trim benchmark_data.xlsx
+run excel_trim_j1 "$qsv_bin" excel --trim -j 1 benchmark_data.xlsx
 run excel_error_format_formula "$qsv_bin" excel --error-format both benchmark_data.xlsx
 run excel_metadata "$qsv_bin" excel --metadata c benchmark_data.xlsx
 run excel_metadata_short "$qsv_bin" excel --metadata s benchmark_data.xlsx
@@ -495,6 +505,8 @@ run fmt_no_final_newline "$qsv_bin" fmt --no-final-newline "$data"
 run foreach "$qsv_bin" foreach City "echo {}" "$data"
 run frequency "$qsv_bin" frequency "$data"
 run --index frequency_index "$qsv_bin" frequency "$data"
+run --index frequency_index_stats_mode_force "$qsv_bin" frequency --stats-mode force "$data"
+run --index frequency_index_stats_mode_none "$qsv_bin" frequency --stats-mode none "$data"
 run frequency_no_limit "$qsv_bin" frequency --limit 0 "$data"
 run --index frequency_no_limit_index "$qsv_bin" frequency --limit 0 "$data"
 run frequency_other_sorted "$qsv_bin" frequency --other-sorted "$data"
@@ -505,14 +517,23 @@ run frequency_ignorecase "$qsv_bin" frequency -i "$data"
 run --index frequency_ignorecase_index "$qsv_bin" frequency -i "$data"
 run frequency_selregex_ignorecase "$qsv_bin" frequency -s /^R/ -i "$data"
 run frequency_j1_ignorecase "$qsv_bin" frequency -j 1 -i "$data"
+run frequency_notrim "$qsv_bin" frequency --no-trim "$data"
+run --index frequency_notrim_index "$qsv_bin" frequency --no-trim "$data"
+run frequency_limit20 "$qsv_bin" frequency --limit 20 "$data"
+run --index frequency_limit20_index "$qsv_bin" frequency --limit 20 "$data"
 run geocode_suggest "$qsv_bin" geocode suggest City --new-column geocoded_city "$data"
+run geocode_suggest_batchall "$qsv_bin" geocode suggest City --new-column geocoded_city --batch 0 "$data"
 run geocode_reverse "$qsv_bin" geocode reverse Location --new-column geocoded_location "$data"
+run geocode_reverse_batchall "$qsv_bin" geocode reverse Location --new-column geocoded_location --batch 0 "$data"
 run index "$qsv_bin" index "$data"
 run input "$qsv_bin" input "$data"
+run json "$qsv_bin" json benchmark_data.json
 run join "$qsv_bin" join \'Community Board\' "$data" community_board communityboards.csv
 run join_casei "$qsv_bin" join \'Community Board\' "$data" community_board --ignore-case communityboards.csv
 run joinp "$qsv_bin" joinp \'Community Board\' "$data" community_board communityboards.csv
+run joinp_streaming "$qsv_bin" joinp \'Community Board\' "$data" community_board communityboards.csv --streaming
 run jsonl "$qsv_bin" jsonl benchmark_data.jsonl
+run jsonl_batchall "$qsv_bin" jsonl benchmark_data.jsonl --batch 0
 run jsonl_j1 "$qsv_bin" jsonl -j 1 benchmark_data.jsonl
 run luau_filter "$qsv_bin" luau filter \"Location == \'\'\" "$data"
 run luau_filter_colidx "$qsv_bin" luau filter --colindex \"Location == \'\'\" "$data"
@@ -588,20 +609,21 @@ run --index split_index "$qsv_bin" split --size 50000 split_tempdir_idx "$data"
 run --index split_index_j1 "$qsv_bin" split --size 50000 -j 1 split_tempdir_idx_j1 "$data"
 run --index split_chunks_index "$qsv_bin" split --chunks 20 split_tempdir_chunks_idx "$data"
 run --index split_chunks_index_j1 "$qsv_bin" split --chunks 20 -j 1 split_tempdir_chunks_idx_j1
-run sqlp "$qsv_bin" sqlp "$data" -Q '"select * from _t_1 where \"Complaint Type\"='\''Noise'\'' and Borough='\''BROOKLYN'\''"'
-run sqlp_aggregations "$qsv_bin" sqlp "$data" -Q '"select Borough, count(*) from _t_1 where \"Complaint Type\"='\''Noise'\'' group by Borough"'
+run sqlp "$qsv_bin" sqlp "$data" -Q --infer-len 100000 '"select * from _t_1 where \"Complaint Type\"='\''Noise'\'' and Borough='\''BROOKLYN'\''"'
+run sqlp_aggregations "$qsv_bin" sqlp "$data" -Q --infer-len 100000 '"select Borough, count(*) from _t_1 where \"Complaint Type\"='\''Noise'\'' group by Borough"'
 run sqlp_aggregations_vs_duckdb duckdb :memory: '"select Borough, count(*) from read_csv_auto('\'''$data''\'') where \"Complaint Type\"='\''Noise'\'' group by Borough"'
-run sqlp_aggregations_expensive "$qsv_bin" sqlp SKIP_INPUT -Q expensive.sql
+run sqlp_aggregations_expensive "$qsv_bin" sqlp SKIP_INPUT -Q --infer-len 100000 expensive.sql
+run sqlp_aggregations_expensive_streaming "$qsv_bin" sqlp SKIP_INPUT -Q --infer-len 100000 --streaming expensive.sql
 run sqlp_aggregations_expensive_vs_duckdb duckdb :memory: -c \".read expensiveduckdb.sql\"
-run sqlp_format_arrow "$qsv_bin" sqlp --format arrow "$data" -Q '"select * from _t_1 where \"Complaint Type\"='\''Noise'\'' and Borough='\''BROOKLYN'\''"'
-run sqlp_format_avro "$qsv_bin" sqlp --format avro "$data" -Q '"select * from _t_1 where \"Complaint Type\"='\''Noise'\'' and Borough='\''BROOKLYN'\''"'
-run sqlp_format_json "$qsv_bin" sqlp --format json "$data" -Q '"select * from _t_1 where \"Complaint Type\"='\''Noise'\'' and Borough='\''BROOKLYN'\''"'
-run sqlp_format_jsonl "$qsv_bin" sqlp --format jsonl "$data" -Q '"select * from _t_1 where \"Complaint Type\"='\''Noise'\'' and Borough='\''BROOKLYN'\''"'
-run sqlp_format_parquet "$qsv_bin" sqlp --format parquet "$data" -Q '"select * from _t_1 where \"Complaint Type\"='\''Noise'\'' and Borough='\''BROOKLYN'\''"'
-run sqlp_format_parquet_statistics "$qsv_bin" sqlp --format parquet --statistics "$data" -Q '"select * from _t_1 where \"Complaint Type\"='\''Noise'\'' and Borough='\''BROOKLYN'\''"'
-run sqlp_lowmemory "$qsv_bin" sqlp "$data" -Q --low-memory '"select * from _t_1 where \"Complaint Type\"='\''Noise'\'' and Borough='\''BROOKLYN'\''"'
-run sqlp_nooptimizations "$qsv_bin" sqlp "$data" -Q --no-optimizations '"select * from _t_1 where \"Complaint Type\"='\''Noise'\'' and Borough='\''BROOKLYN'\''"'
-run sqlp_tryparsedates "$qsv_bin" sqlp "$data" -Q --try-parsedates '"select * from _t_1 where \"Complaint Type\"='\''Noise'\'' and Borough='\''BROOKLYN'\''"'
+run sqlp_format_arrow "$qsv_bin" sqlp --format arrow "$data" -Q --infer-len 100000 '"select * from _t_1 where \"Complaint Type\"='\''Noise'\'' and Borough='\''BROOKLYN'\''"'
+run sqlp_format_avro "$qsv_bin" sqlp --format avro "$data" -Q --infer-len 100000 '"select * from _t_1 where \"Complaint Type\"='\''Noise'\'' and Borough='\''BROOKLYN'\''"'
+run sqlp_format_json "$qsv_bin" sqlp --format json "$data" -Q --infer-len 100000 '"select * from _t_1 where \"Complaint Type\"='\''Noise'\'' and Borough='\''BROOKLYN'\''"'
+run sqlp_format_jsonl "$qsv_bin" sqlp --format jsonl "$data" -Q --infer-len 100000 '"select * from _t_1 where \"Complaint Type\"='\''Noise'\'' and Borough='\''BROOKLYN'\''"'
+run sqlp_format_parquet "$qsv_bin" sqlp --format parquet "$data" -Q --infer-len 100000 '"select * from _t_1 where \"Complaint Type\"='\''Noise'\'' and Borough='\''BROOKLYN'\''"'
+run sqlp_format_parquet_statistics "$qsv_bin" sqlp --format parquet --statistics "$data" -Q --infer-len 100000 '"select * from _t_1 where \"Complaint Type\"='\''Noise'\'' and Borough='\''BROOKLYN'\''"'
+run sqlp_lowmemory "$qsv_bin" sqlp "$data" -Q --low-memory --infer-len 100000 '"select * from _t_1 where \"Complaint Type\"='\''Noise'\'' and Borough='\''BROOKLYN'\''"'
+run sqlp_nooptimizations "$qsv_bin" sqlp "$data" -Q --no-optimizations --infer-len 100000 '"select * from _t_1 where \"Complaint Type\"='\''Noise'\'' and Borough='\''BROOKLYN'\''"'
+run sqlp_tryparsedates "$qsv_bin" sqlp "$data" -Q --try-parsedates --infer-len 100000 '"select * from _t_1 where \"Complaint Type\"='\''Noise'\'' and Borough='\''BROOKLYN'\''"'
 run sqlp_tryparsedates_inferlen "$qsv_bin" sqlp "$data" -Q --infer-len 10000 --try-parsedates '"select * from _t_1 where \"Complaint Type\"='\''Noise'\'' and Borough='\''BROOKLYN'\''"'
 run stats "$qsv_bin" stats --force "$data"
 run stats_create_cache "$qsv_bin" stats --force "$data"
@@ -625,6 +647,7 @@ run to_sqlite "$qsv_bin" to sqlite benchmark_work.db "$data"
 run to_parquet "$qsv_bin" to parquet benchmark_work "$data"
 run to_datapackage "$qsv_bin" to datapackage benchmark_work.json "$data"
 run tojsonl "$qsv_bin" tojsonl "$data"
+run tojsonl_batchall "$qsv_bin" tojsonl --batch 0 "$data"
 run tojsonl_j1 "$qsv_bin" tojsonl -j 1 "$data"
 run tojsonl_trim "$qsv_bin" tojsonl --trim "$data"
 run tojsonl_trim_j1 "$qsv_bin" tojsonl --trim -j 1 "$data"
@@ -633,9 +656,11 @@ run --index tojsonl_index_j1 "$qsv_bin" tojsonl --jobs 1 "$data"
 run transpose "$qsv_bin" transpose "$data"
 run transpose_multipass "$qsv_bin" transpose --multipass "$data"
 run validate "$qsv_bin" validate "$data" "$schema"
+run validate_batchall "$qsv_bin" validate --batch 0 "$data" "$schema"
 run validate_no_schema "$qsv_bin" validate "$data"
 run validate_valid_output "$qsv_bin" validate "$data" "$schema" --valid-output -
 run --index validate_index "$qsv_bin" validate "$data" "$schema"
+run --index validate_batchall_index "$qsv_bin" validate --batch 0 "$data" "$schema"
 run --index validate_no_schema_index "$qsv_bin" validate "$data"
 run --index validate_valid_output_index "$qsv_bin" validate "$data" "$schema" --valid-output -
 
