@@ -18,6 +18,12 @@ fixlengths options:
     -l, --length <arg>     Forcefully set the length of each record. If a
                            record is not the size given, then it is truncated
                            or expanded as appropriate.
+    -i, --insert <pos>     If empty fields need to be inserted, insert them
+                           at <pos>. If <pos> is zero, then it is inserted
+                           at the end of each record. If <pos> is negative, it
+                           is inserted from the END of each record going backwards.
+                           If <pos> is positive, it is inserted from the BEGINNING
+                           of each record going forward. [default: 0]
 
 Common options:
     -h, --help             Display this message
@@ -39,6 +45,7 @@ use crate::{
 struct Args {
     arg_input:      Option<String>,
     flag_length:    Option<usize>,
+    flag_insert:    i16,
     flag_output:    Option<String>,
     flag_delimiter: Option<Delimiter>,
 }
@@ -51,12 +58,14 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         .flexible(true);
     let length = if let Some(length) = args.flag_length {
         if length == 0 {
-            return fail!("Length must be greater than 0.");
+            return fail_incorrectusage_clierror!("Length must be greater than 0.");
         }
         length
     } else {
         if config.is_stdin() {
-            return fail!("<stdin> cannot be used in this command. Please specify a file path.");
+            return fail_incorrectusage_clierror!(
+                "<stdin> cannot be used in this command. Please specify a file path."
+            );
         }
         let mut maxlen = 0_usize;
         let mut rdr = config.reader()?;
@@ -75,17 +84,50 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
     let mut rdr = config.reader()?;
     let mut wtr = Config::new(&args.flag_output).writer()?;
-    for r in rdr.byte_records() {
-        let mut r = r?;
-        if length >= r.len() {
-            for _ in r.len()..length {
-                r.push_field(b"");
+    let mut record = csv::ByteRecord::new();
+    let mut record_work = csv::ByteRecord::new();
+    #[allow(unused_assignments)]
+    let mut field_idx = 1_i16;
+
+    let insert_pos = if args.flag_insert < 0 {
+        length as i16 + args.flag_insert
+    } else {
+        args.flag_insert
+    };
+    log::debug!("length: {length} insert_pos: {insert_pos}");
+
+    while rdr.read_byte_record(&mut record)? {
+        if length >= record.len() {
+            if args.flag_insert == 0 {
+                for _ in record.len()..length {
+                    record.push_field(b"");
+                }
+            } else {
+                record_work.clear();
+                field_idx = 1_i16;
+                for field in &record {
+                    if field_idx == insert_pos {
+                        // insert all the empty fields at the insert position
+                        for _ in record.len()..length {
+                            record_work.push_field(b"");
+                        }
+                    }
+                    record_work.push_field(field);
+
+                    field_idx += 1;
+                }
+                if record_work.len() <= length {
+                    // insert all the empty fields at the end
+                    for _ in record_work.len()..length {
+                        record_work.push_field(b"");
+                    }
+                }
+                record = record_work.clone();
             }
         } else {
-            r.truncate(length);
+            record.truncate(length);
         }
-        wtr.write_byte_record(&r)?;
+        wtr.write_byte_record(&record)?;
     }
-    wtr.flush()?;
-    Ok(())
+    Ok(wtr.flush()?)
 }
