@@ -6,15 +6,17 @@ Polars SQL is a SQL dialect, converting SQL queries to fast Polars LazyFrame exp
 (see https://docs.pola.rs/user-guide/sql/intro/).
 
 For a list of SQL functions and keywords supported by Polars SQL, see
-https://github.com/pola-rs/polars/blob/rs-0.40.0/crates/polars-sql/src/functions.rs
-https://github.com/pola-rs/polars/blob/rs-0.40.0/crates/polars-sql/src/keywords.rs and
-https://github.com/pola-rs/polars/issues/7227
+https://github.com/pola-rs/polars/blob/rs-0.41.3/crates/polars-sql/src/functions.rs and
+https://github.com/pola-rs/polars/blob/rs-0.41.3/crates/polars-sql/src/keywords.rs.
+https://docs.pola.rs/py-polars/html/reference/sql/index.html also provides a more readable
+version of the SQL functions and keywords, though be aware that it's for the Python version
+of Polars, so there will be some minor syntax differences.
 
 Returns the shape of the query result (number of rows, number of columns) to stderr.
 
 Example queries:
 
-   qsv sqlp data.csv 'select * from data where col1 > 10 order by col2 desc limit 20'
+   qsv sqlp data.csv 'select * from data where col1 > 10 order by all desc limit 20'
 
    qsv sqlp data.csv 'select col1, col2 as friendlyname from data' --format parquet --output data.parquet
 
@@ -25,6 +27,7 @@ Example queries:
 
    qsv sqlp data.csv data2.csv 'SELECT col1 FROM data WHERE col1 IN (SELECT col2 FROM data2)'
 
+  # Unions and Joins are supported.
    qsv sqlp data1.csv data2.csv 'SELECT * FROM data1 UNION ALL BY NAME SELECT * FROM data2'
 
    qsv sqlp tbl_a.csv tbl_b.csv tbl_c.csv "SELECT * FROM tbl_a \
@@ -44,6 +47,10 @@ Example queries:
 
    qsv sqlp tbl1.csv "SELECT x FROM tbl1 WHERE x IN (SELECT y FROM tbl1)"
 
+  # Natural Joins are supported too! (https://www.w3resource.com/sql/joins/natural-join.php)
+   qsv sqlp data1.csv data2.csv data3.csv \
+    "SELECT COLUMNS('^[^:]+$') FROM data1 NATURAL JOIN data2 NATURAL JOIN data3 ORDER BY COMPANY_ID",
+
   # Use a SQL script to run a long, complex SQL query or to run SEVERAL SQL queries.
   # When running several queries, each query needs to be separated by a semicolon,
   # the last query will be returned as the result.
@@ -69,6 +76,19 @@ Example queries:
   # https://en.wikipedia.org/wiki/Three-way_comparison#Spaceship_operator
    qsv sqlp data.csv data2.csv "select data.c2 <=> data2.c2 from data join data2 on data.c1 = data2.c1"
 
+  # support ^@ ("starts with"), and ~~ (like) ,~~* (ilike),!~~ (not like),!~~* (not ilike) operators
+    qsv sqlp data.csv "select * from data WHERE col1 ^@ 'foo'"
+    qsv sqlp data.csv "select c1 ^@ 'a' AS c1_starts_with_a from data"
+    qsv sqlp data.csv "select c1 ~~* '%B' AS c1_ends_with_b_caseinsensitive from data"
+
+  # support SELECT * ILIKE wildcard syntax
+    # select all columns from customers where the column contains 'a' followed by an 'e'
+    # with any characters (or no characters), in between, case-insensitive
+    # if customers.csv has columns LastName, FirstName, Address, City, State, Zip
+    # this query will return all columns for all rows except the columns that don't
+    # contain 'a' followed by an 'e' - i.e. except City and Zip
+    qsv sqlp customers.csv "SELECT * ILIKE '%a%e%' FROM customers ORDER BY LastName, FirstName"
+
   # regex operators: "~" (contains pattern, case-sensitive); "~*" (contains pattern, case-insensitive)
   #   "!~" (does not contain pattern, case-sensitive); "!~*" (does not contain pattern, case-insensitive)
    qsv sqlp data.csv "select * from data WHERE col1 ~ '^foo' AND col2 > 10"
@@ -92,17 +112,23 @@ Example queries:
    qsv sqlp data.csv "select * from data join read_parquet('data2.parquet') as t2 ON data.c1 = t2.c1"
    qsv sqlp data.csv "select * from data join read_ndjson('data2.jsonl') as t2 on data.c1 = t2.c1"
    qsv sqlp data.csv "select * from data join read_ipc('data2.arrow') as t2 ON data.c1 = t2.c1"
+   qsv sqlp SKIP_INPUT "select * from read_parquet('data.parquet') order by col1 desc limit 100"
+   qsv sqlp SKIP_INPUT "select * from read_ndjson('data.jsonl') as t1 join read_ipc('data.arrow') as t2 on t1.c1 = t2.c1" 
 
   # you can also directly load CSVs using the Polars read_csv() SQL function. This is useful when
-  # you want to bypass the regular CSV parser and use Polars' multithreaded, mem-mapped CSV parser
-  # instead - making for dramatically faster queries at the cost of CSV parser configurability.
-  # (i.e. limited to comma delimiter, no CSV comments, etc.)
-   qsv sqlp small_dummy.csv "select * from read_csv('data.csv') order by col1 desc limit 100"
+  # you want to bypass the regular CSV parser (with SKIP_INPUT) and use Polars' multithreaded,
+  # mem-mapped CSV parser instead - making for dramatically faster queries at the cost of CSV parser
+  # configurability (i.e. limited to comma delimiter, no CSV comments, etc.).
+   qsv sqlp SKIP_INPUT "select * from read_csv('data.csv') order by col1 desc limit 100"
 
-  Note that sqlp will automatically use this "fast path" optimization when there is only 
-  one input CSV file, no CSV parsing options are used, its not a SQL script and the
+  Note that sqlp will automatically use this "fast path" read_csv() optimization when there 
+  is only one input CSV file, no CSV parsing options are used, its not a SQL script and the
   `--no-optimizations` flag is not set.
 
+  # apart from using Polar's table functions, you can also use SKIP_INPUT when the SELECT
+  # statement doesn't require an input file
+   qsv sqlp SKIP_INPUT "SELECT 1 AS one, '2' AS two, 3.0 AS three"
+
   # use stdin as input
    cat data.csv | qsv sqlp - 'select * from stdin'
    cat data.csv | qsv sqlp - data2.csv 'select * from stdin join data2 on stdin.col1 = data2.col1'
@@ -157,11 +183,12 @@ sqlp options:
                               that you may want to try to set `--ignore-errors` to relax
                               the CSV parsing of dates.
     --infer-len <arg>         The number of rows to scan when inferring the schema of the CSV.
-                              Set to 0 to do a full table scan (warning: very slow).
-                              [default: 1000]
+                              Set to 0 to do a full table scan (warning: can be slow).
+                              [default: 10000]
+    --streaming               Use streaming mode when parsing CSVs. This will use less memory
+                              but will be slower. Only use this when you get out of memory errors.
     --low-memory              Use low memory mode when parsing CSVs. This will use less memory
-                              but will be slower. It will also process LazyFrames in streaming mode.
-                              Only use this when you get out of memory errors.
+                              but will be slower. Only use this when you get out of memory errors.
     --no-optimizations        Disable non-default query optimizations. This will make queries slower.
                               Use this when you get query errors or to force CSV parsing when there
                               is only one input file, no CSV parsing options are used and its not
@@ -236,8 +263,8 @@ use polars::{
     io::avro::{AvroWriter, Compression as AvroCompression},
     prelude::{
         CsvWriter, DataFrame, GzipLevel, IpcCompression, IpcWriter, JsonFormat, JsonWriter,
-        LazyCsvReader, LazyFileListReader, NullValues, ParquetCompression, ParquetWriter,
-        SerWriter, ZstdLevel,
+        LazyCsvReader, LazyFileListReader, NullValues, OptState, ParquetCompression, ParquetWriter,
+        SerWriter, StatisticsOptions, ZstdLevel,
     },
     sql::SQLContext,
 };
@@ -245,8 +272,8 @@ use regex::Regex;
 use serde::Deserialize;
 
 use crate::{
-    cmd::joinp::tsvtab_delim,
-    config::{Delimiter, DEFAULT_WTR_BUFFER_CAPACITY},
+    cmd::joinp::tsvssv_delim,
+    config::{Config, Delimiter, DEFAULT_WTR_BUFFER_CAPACITY},
     util,
     util::process_input,
     CliResult,
@@ -262,6 +289,7 @@ struct Args {
     flag_format:                String,
     flag_try_parsedates:        bool,
     flag_infer_len:             usize,
+    flag_streaming:             bool,
     flag_low_memory:            bool,
     flag_no_optimizations:      bool,
     flag_ignore_errors:         bool,
@@ -316,7 +344,7 @@ impl OutputMode {
 
             let w = match args.flag_output {
                 Some(path) => {
-                    delim = tsvtab_delim(path.clone(), delim);
+                    delim = tsvssv_delim(path.clone(), delim);
                     Box::new(File::create(path)?) as Box<dyn Write>
                 },
                 None => Box::new(io::stdout()) as Box<dyn Write>,
@@ -364,9 +392,25 @@ impl OutputMode {
                         },
                     };
 
+                    let statistics_options = if args.flag_statistics {
+                        StatisticsOptions {
+                            min_value:      true,
+                            max_value:      true,
+                            distinct_count: true,
+                            null_count:     true,
+                        }
+                    } else {
+                        StatisticsOptions {
+                            min_value:      false,
+                            max_value:      false,
+                            distinct_count: false,
+                            null_count:     false,
+                        }
+                    };
+
                     ParquetWriter::new(&mut w)
                         .with_row_group_size(Some(768 ^ 2))
-                        .with_statistics(args.flag_statistics)
+                        .with_statistics(statistics_options)
                         .with_compression(parquet_compression)
                         .finish(&mut df)
                         .map(|_| ())
@@ -550,30 +594,40 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         None
     };
 
-    let optimization_state = if args.flag_no_optimizations {
-        // use default optimization state
-        polars::lazy::frame::OptState {
-            file_caching: !args.flag_low_memory,
-            streaming: args.flag_low_memory,
-            ..Default::default()
-        }
-    } else {
-        polars::lazy::frame::OptState {
-            projection_pushdown:  true,
-            predicate_pushdown:   true,
-            cluster_with_columns: true,
-            type_coercion:        true,
-            simplify_expr:        true,
-            file_caching:         !args.flag_low_memory,
-            slice_pushdown:       true,
-            comm_subplan_elim:    !args.flag_low_memory,
-            comm_subexpr_elim:    true,
-            streaming:            args.flag_low_memory,
-            fast_projection:      true,
-            eager:                false,
-            row_estimate:         true,
-        }
-    };
+    let mut optimization_state = polars::lazy::frame::OptState::default();
+    optimization_state |= OptState::PROJECTION_PUSHDOWN
+        | OptState::PREDICATE_PUSHDOWN
+        | OptState::CLUSTER_WITH_COLUMNS
+        | OptState::TYPE_COERCION
+        | OptState::SIMPLIFY_EXPR
+        | OptState::FILE_CACHING
+        | OptState::SLICE_PUSHDOWN
+        | OptState::COMM_SUBEXPR_ELIM
+        | OptState::FAST_PROJECTION
+        | OptState::ROW_ESTIMATE;
+
+    if args.flag_streaming {
+        optimization_state |= OptState::NEW_STREAMING;
+    }
+
+    // check if the input is a SQL script (ends with .sql)
+    let is_sql_script = std::path::Path::new(&args.arg_sql)
+        .extension()
+        .map_or(false, |ext| ext.eq_ignore_ascii_case("sql"));
+
+    // if infer_len is 0, its not a SQL script, and there is only one input CSV, we can infer the
+    // schema of the CSV more intelligently by counting the number of rows in the file instead of
+    // scanning the entire file with a 0 infer_len which triggers a full table scan.
+    args.flag_infer_len =
+        if args.flag_infer_len == 0 && !is_sql_script && !skip_input && args.arg_input.len() == 1 {
+            let rconfig = Config::new(&Some(args.arg_input[0].to_string_lossy().to_string()))
+                .delimiter(args.flag_delimiter)
+                .no_headers(false);
+            util::count_rows(&rconfig).unwrap_or(0) as usize
+        } else {
+            args.flag_infer_len
+        };
+
     // gated by log::log_enabled!(log::Level::Debug) to avoid the
     // relatively expensive overhead of generating the debug string
     // for the optimization state struct
@@ -583,7 +637,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         log::debug!(
             "Delimiter: {delim} Infer_schema_len: {infer_len} try_parse_dates: {parse_dates} \
              ignore_errors: {ignore_errors}, low_memory: {low_memory}, float_precision: \
-             {float_precision:?}, skip_input: {skip_input}",
+             {float_precision:?}, skip_input: {skip_input}, is_sql_script: {is_sql_script}",
             infer_len = args.flag_infer_len,
             parse_dates = args.flag_try_parsedates,
             ignore_errors = args.flag_ignore_errors,
@@ -592,11 +646,6 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         );
     }
 
-    // check if the input is a SQL script (ends with .sql)
-    let is_sql_script = std::path::Path::new(&args.arg_sql)
-        .extension()
-        .map_or(false, |ext| ext.eq_ignore_ascii_case("sql"));
-
     let mut ctx = SQLContext::new();
     let mut table_aliases = HashMap::with_capacity(args.arg_input.len());
     let mut lossy_table_name = Cow::default();
@@ -618,7 +667,8 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         && delim == b','
         && !args.flag_no_optimizations
         && !args.flag_try_parsedates
-        && args.flag_infer_len != 1000
+        && args.flag_infer_len != 10_000
+        && !args.flag_streaming
         && !args.flag_low_memory
         && !args.flag_truncate_ragged_lines
         && !args.flag_ignore_errors
@@ -684,7 +734,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                 .with_missing_is_null(true)
                 .with_comment_prefix(comment_char.as_deref())
                 .with_null_values(Some(NullValues::AllColumns(rnull_values.clone())))
-                .with_separator(tsvtab_delim(table, delim))
+                .with_separator(tsvssv_delim(table, delim))
                 .with_infer_schema_length(Some(args.flag_infer_len))
                 .with_try_parse_dates(args.flag_try_parsedates)
                 .with_ignore_errors(args.flag_ignore_errors)
