@@ -1,15 +1,15 @@
 static USAGE: &str = r#"
 Compute summary statistics & infers data types for each column in a CSV. 
 
-Summary statistics includes sum, min/max/range, min/max length, mean, stddev, variance,
-nullcount, max_precision, sparsity, quartiles, interquartile range (IQR), lower/upper fences,
-skewness, median, cardinality, mode/s & "antimode/s", and median absolute deviation (MAD).
-Note that some statistics require loading the entire file into memory, so they must be
-enabled explicitly. 
+Summary statistics includes sum, min/max/range, min/max length, mean, standard error of the mean (SEM),
+stddev, variance, coefficient of variation (CV), nullcount, max_precision, sparsity, quartiles,
+interquartile range (IQR), lower/upper fences, skewness, median, cardinality, mode/s & "antimode/s", 
+and median absolute deviation (MAD). Note that some statistics require loading the entire file into
+memory, so they must be enabled explicitly. 
 
 By default, the following "streaming" statistics are reported for *every* column:
-sum, min/max/range values, min/max length, mean, stddev, variance, nullcount, max_precision and
-sparsity. The default set of statistics corresponds to ones that can be computed efficiently
+sum, min/max/range values, min/max length, mean, sem, stddev, variance, cv, nullcount, max_precision
+and sparsity. The default set of statistics corresponds to ones that can be computed efficiently
 on a stream of data (i.e., constant memory) and works with arbitrarily large CSV files.
 
 The following additional "non-streaming" statistics require loading the entire file into memory:
@@ -61,12 +61,61 @@ hasn't changed, the stats will be loaded from the cache instead of recomputing i
 
 These cached stats are also used by other qsv commands (currently `schema` & `tojsonl`) to
 load the stats into memory faster. If the cached stats are not current (i.e., the input file
-is newer than the cached stats), the cached stats will be ignored and recomputed.
-
-For examples, see the "boston311" test files in 
-https://github.com/jqnatividad/qsv/tree/master/resources/test and
+is newer than the cached stats), the cached stats will be ignored and recomputed. For example,
+see the "boston311" test files in 
 https://github.com/jqnatividad/qsv/blob/4529d51273218347fef6aca15ac24e22b85b2ec4/tests/test_stats.rs#L608.
 
+Examples:
+
+Compute "streaming" statistics for the "nyc311.csv" file:
+   $ qsv stats nyc311.csv
+
+Compute all statistics for the "nyc311.csv" file:
+    $ qsv stats --everything nyc311.csv
+    $ qsv stats -E nyc311.csv
+
+Compute all statistics for the "nyc311.csv", inferring dates using default date column name patterns:
+    $ qsv stats -E --infer-dates nyc311.csv
+
+Compute all statistics for the "nyc311.csv", inferring dates only for columns with "_date" & "_dte":
+    $ qsv stats -E --infer-dates --dates-whitelist _date,_dte nyc311.csv
+
+In addition, also infer boolean data types for the "nyc311.csv" file:
+    $ qsv stats -E --infer-dates --dates-whitelist _date --infer-boolean nyc311.csv
+
+In addition to basis "streaming" stats, also compute the cardinality for the "nyc311.csv" file:
+    $ qsv stats --cardinality nyc311.csv
+
+Prefer DMY format when inferring dates for the "nyc311.csv" file:
+    $ qsv stats -E --infer-dates --prefer-dmy nyc311.csv    
+
+Infer data types only for the "nyc311.csv" file:
+    $ qsv stats --typesonly nyc311.csv
+
+Infer data types only, including boolean and date types for the "nyc311.csv" file:
+    $ qsv stats --typesonly --infer-boolean --infer-dates nyc311.csv
+
+Automatically create an index for the "nyc311.csv" file if it's larger than 5MB
+and there is no existing index file:
+    $ qsv stats -E --cache-threshold -5000000 nyc311.csv
+
+Auto-create an index for the "nyc311.csv" file if it's larger than 5MB and delete the index
+and the stats cache file after the stats run:
+    $ qsv stats -E --cache-threshold -5000005 nyc311.csv
+
+Prompt for CSV/TSV/TAB file to compute stats for:
+    $ qsv prompt -F tsv,csv,tab | qsv stats -E | qsv table
+
+Prompt for a file to save the stats to in the ~/Documents directory:
+    $ qsv stats -E nyc311.csv | qsv prompt -d ~/Documents --fd-output
+
+Prompt for both INPUT and OUTPUT files in the ~/Documents dir with custom prompts:
+    $ qsv prompt -m 'Select a CSV file to summarize' -d ~/Documents -F csv | \
+      qsv stats -E --infer-dates | \
+      qsv prompt -m 'Save summary to...' -d ~/Documents --fd-output --save-fname summarystats.csv
+
+For more examples, see https://github.com/jqnatividad/qsv/tree/master/resources/test
+
 Usage:
     qsv stats [options] [<input>]
     qsv stats --help
@@ -134,12 +183,12 @@ stats options:
                               Note that a file handle is opened for each job.
                               When not set, the number of jobs is set to the
                               number of CPUs detected.
-    --stats-binout            Write the stats in binary format. This is used internally
-                              by other qsv commands (currently `schema` & `tojsonl`) to
-                              load cached stats into memory faster. If set, the snappy compressed
+    --stats-binout            Write the stats in binary format. This is used internally by other
+                              qsv commands (currently `frequency`, `schema` & `tojsonl`) to load
+                              cached stats into memory faster. If set, the snappy compressed
                               binary encoded stats will be written to <FILESTEM>.stats.csv.bin.sz.
-                              You can preemptively create the binary encoded stats file
-                              by using this option BEFORE running the `schema` and `tojsonl`
+                              You can preemptively create the binary encoded stats file by using
+                              this option BEFORE running the `frequency`, `schema` & `tojsonl`
                               commands and they will automatically load the binary encoded
                               stats file if it exists.
  -c, --cache-threshold <arg>  When greater than 1, the threshold in milliseconds before caching
@@ -282,7 +331,7 @@ const MS_IN_DAY_INT: i64 = 86_400_000;
 const DAY_DECIMAL_PLACES: u32 = 5;
 
 // maximum number of output columns
-const MAX_STAT_COLUMNS: usize = 32;
+const MAX_STAT_COLUMNS: usize = 34;
 
 // maximum number of antimodes to display
 const MAX_ANTIMODES: usize = 10;
@@ -753,12 +802,13 @@ impl Args {
                     .unwrap();
             });
         }
-        assert!(results.len() == records.len());
         for (i, recv) in results.into_iter().enumerate() {
-            // safety: the assert above guarantees that records index access is safe and
-            // doesn't require a bounds check.
+            // safety: results.len() == records.len() so we know the index is valid
+            // and doesn't require a bounds check.
             // The unwrap on recv.recv() is safe as the channel is bounded
-            records[i] = recv.recv().unwrap();
+            unsafe {
+                *records.get_unchecked_mut(i) = recv.recv().unwrap();
+            }
         }
         records
     }
@@ -842,7 +892,7 @@ impl Args {
             return csv::StringRecord::from(vec!["field", "type"]);
         }
 
-        // with --everything, we have 32 columns at most
+        // with --everything, we have 34 columns at most
         let mut fields = Vec::with_capacity(MAX_STAT_COLUMNS);
         fields.extend_from_slice(&[
             "field",
@@ -855,8 +905,10 @@ impl Args {
             "min_length",
             "max_length",
             "mean",
+            "sem",
             "stddev",
             "variance",
+            "cv",
             "nullcount",
             "max_precision",
             "sparsity",
@@ -1174,7 +1226,7 @@ impl Stats {
 
         let typ = self.typ;
         // prealloc memory for performance
-        // we have 32 columns at most with --everything
+        // we have 34 columns at most with --everything
         let mut pieces = Vec::with_capacity(MAX_STAT_COLUMNS);
 
         let empty = String::new;
@@ -1333,32 +1385,45 @@ impl Stats {
             pieces.extend_from_slice(&[empty(), empty()]);
         }
 
-        // mean, stddev & variance
+        // mean, sem, stddev, variance & cv
         if typ == TString || typ == TNull {
-            pieces.extend_from_slice(&[empty(), empty(), empty()]);
+            pieces.extend_from_slice(&[empty(), empty(), empty(), empty(), empty()]);
         } else if let Some(ref v) = self.online {
+            let std_dev = v.stddev();
+            #[allow(clippy::cast_precision_loss)]
+            let sem = std_dev / (v.len() as f64).sqrt();
+            let mean = v.mean();
+            let cv = (std_dev / mean) * 100_f64;
             if self.typ == TFloat || self.typ == TInteger {
                 pieces.extend_from_slice(&[
-                    util::round_num(v.mean(), round_places),
-                    util::round_num(v.stddev(), round_places),
+                    util::round_num(mean, round_places),
+                    util::round_num(sem, round_places),
+                    util::round_num(std_dev, round_places),
                     util::round_num(v.variance(), round_places),
+                    util::round_num(cv, round_places),
                 ]);
             } else {
-                pieces.push(timestamp_ms_to_rfc3339(v.mean() as i64, typ));
-                // instead of returning stdev in seconds, let's return it in
-                // days as it easier to handle
+                // by the time we get here, the type is a TDateTime or TDate
+                pieces.push(timestamp_ms_to_rfc3339(mean as i64, typ));
+                // instead of returning sem, stdev & variance as timestamps, return it in
+                // days as its more human readable and practical for real-world use cases
                 // Round to at least 5 decimal places, so we have millisecond precision
                 pieces.push(util::round_num(
-                    v.stddev() / MS_IN_DAY,
+                    sem / MS_IN_DAY,
+                    u32::max(round_places, DAY_DECIMAL_PLACES),
+                ));
+                pieces.push(util::round_num(
+                    std_dev / MS_IN_DAY,
                     u32::max(round_places, DAY_DECIMAL_PLACES),
                 ));
                 pieces.push(util::round_num(
                     v.variance() / (MS_IN_DAY * MS_IN_DAY),
                     u32::max(round_places, DAY_DECIMAL_PLACES),
                 ));
+                pieces.push(util::round_num(cv, round_places));
             }
         } else {
-            pieces.extend_from_slice(&[empty(), empty(), empty()]);
+            pieces.extend_from_slice(&[empty(), empty(), empty(), empty(), empty()]);
         }
 
         // nullcount
@@ -1544,11 +1609,11 @@ enum FieldType {
 }
 
 impl FieldType {
-    // infer data type from a given sample & current type inference
-    // infer_dates signals if date inference should be attempted
-    // returns the inferred type and if infer_dates is true,
-    // the date in ms since the epoch if the type is a date or datetime
-    // otherwise, None
+    /// infer data type from a given sample & current type inference
+    /// infer_dates signals if date inference should be attempted
+    /// returns the inferred type and if infer_dates is true,
+    /// the date in ms since the epoch if the type is a date or datetime
+    /// otherwise, None
     #[inline]
     pub fn from_sample(
         infer_dates: bool,
@@ -1558,72 +1623,52 @@ impl FieldType {
     ) -> (FieldType, Option<i64>) {
         // faster than sample.len() == 0 or sample.is_empty() per microbenchmarks
         if b"" == sample {
-            return (TNull, None);
+            return (FieldType::TNull, None);
         }
+
         // no need to do type checking if current_type is already a String
         if current_type == FieldType::TString {
             return (FieldType::TString, None);
         }
 
-        let utf8_string = if current_type == FieldType::TInteger
-            || current_type == FieldType::TFloat
-            || current_type == FieldType::TNull
-        {
-            if let Ok(int_val) = atoi_simd::parse::<i64>(sample) {
-                // safety: we know sample is not empty
-                if int_val != 0 && unsafe { sample.get_unchecked(0) == &b'0' } {
-                    // leading zero, its a string (e.g. zip codes)
-                    return (TString, None);
+        if let Ok(s) = simdutf8::basic::from_utf8(sample) {
+            // Check for integer, with leading zero check for strings like zip codes
+            if atoi_simd::parse::<i64>(s.as_bytes()).is_ok() {
+                if s == "0" || !s.starts_with('0') {
+                    return (FieldType::TInteger, None);
                 }
-                return (TInteger, None);
-            }
-
-            let Ok(string) = from_utf8(sample) else {
-                // if the string is not valid utf8, we assume it is a binary string
-                // and return a string type
+                // If starts with '0' but not "0", it's a string
                 return (FieldType::TString, None);
-            };
-
-            if string.parse::<f64>().is_ok() {
-                return (TFloat, None);
             }
 
-            Some(string)
+            // Check for float
+            if s.parse::<f64>().is_ok() {
+                return (FieldType::TFloat, None);
+            }
+
+            // Check for date/datetime if infer_dates is true
+            if infer_dates {
+                if let Ok(parsed_date) = parse_with_preference(s, prefer_dmy) {
+                    let ts_val = parsed_date.timestamp_millis();
+                    // check if the tstamp (Unix Epoch format) modulo by 86400000 (ms in a day)
+                    // if the remainder is 0, we says its Date type candidate, otherwise, its
+                    // DateTime. this is a performance optimization to avoid
+                    // parsing the date to rfc3339 and then checking if the time
+                    // component is T00:00:00, as was done previously
+                    // see https://stackoverflow.com/questions/40948290/is-it-safe-to-use-modulo-operator-with-unix-epoch-timestamp
+                    if ts_val % MS_IN_DAY_INT == 0 {
+                        return (FieldType::TDate, Some(ts_val));
+                    }
+                    return (FieldType::TDateTime, Some(ts_val));
+                }
+            }
         } else {
-            None
+            // If not valid UTF-8, it's a binary string, return as TString
+            return (FieldType::TString, None);
         };
 
-        if infer_dates
-            && (current_type == FieldType::TDate
-                || current_type == FieldType::TDateTime
-                || current_type == FieldType::TNull)
-        {
-            // if we already have a utf8 string, use it, otherwise, convert sample to utf8
-            // if the sample isn't valid utf8, we assume a binary string & return a string type
-            let date_string = if let Some(s) = utf8_string {
-                s
-            } else if let Ok(s) = from_utf8(sample) {
-                s
-            } else {
-                // the string is not valid utf8, we assume it is a binary string
-                // and return a string type
-                return (FieldType::TString, None);
-            };
-
-            if let Ok(parsed_date) = parse_with_preference(date_string, prefer_dmy) {
-                // check if the tstamp (Unix Epoch format) modulo by 86400000 (ms in a day)
-                // if the remainder is 0, we says its Date type candidate, otherwise, its DateTime.
-                // this is a performance optimization to avoid parsing the date to rfc3339
-                // and then checking if the time component is T00:00:00, as was done previously
-                // see https://stackoverflow.com/questions/40948290/is-it-safe-to-use-modulo-operator-with-unix-epoch-timestamp
-                let ts_val = parsed_date.timestamp_millis();
-                if ts_val % MS_IN_DAY_INT == 0 {
-                    return (TDate, Some(ts_val));
-                }
-                return (TDateTime, Some(ts_val));
-            }
-        }
-        (TString, None)
+        // Default to TString if none of the above conditions are met
+        (FieldType::TString, None)
     }
 }
 
