@@ -29,7 +29,7 @@ const NO_INDEX_WARNING_FILESIZE: u64 = 100_000_000; // 100MB
 // so we don't have to keep checking if the index has been created
 static AUTO_INDEXED: AtomicBool = AtomicBool::new(false);
 
-#[derive(Clone, Copy, Debug, PartialEq)]
+#[derive(Clone, Copy, Debug, PartialEq, Eq)]
 pub struct Delimiter(pub u8);
 
 /// Delimiter represents values that can be passed from the command line that
@@ -95,7 +95,9 @@ pub struct Config {
 }
 
 // Empty trait as an alias for Seek and Read that avoids auto trait errors
+#[cfg(any(feature = "feature_capable", feature = "lite"))]
 pub trait SeekRead: io::Seek + io::Read {}
+#[cfg(any(feature = "feature_capable", feature = "lite"))]
 impl<T: io::Seek + io::Read> SeekRead for T {}
 
 impl Config {
@@ -134,6 +136,8 @@ impl Config {
                 let mut snappy = false;
                 let delim = if file_extension == "tsv" || file_extension == "tab" {
                     b'\t'
+                } else if file_extension == "ssv" {
+                    b';'
                 } else if file_extension == "csv" {
                     b','
                 } else {
@@ -144,6 +148,9 @@ impl Config {
                     } else if filename.ends_with(".tsv.sz") || filename.ends_with(".tab.sz") {
                         snappy = true;
                         b'\t'
+                    } else if filename.ends_with(".ssv.sz") {
+                        snappy = true;
+                        b';'
                     } else {
                         default_delim
                     }
