@@ -36,7 +36,7 @@ fn frequency_no_headers() {
 
     let mut got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
     got = got.into_iter().skip(1).collect();
-    got.sort();
+    got.sort_unstable();
     let expected = vec![
         svec!["1", "(NULL)", "1", "12.5"],
         svec!["1", "(NULL)", "1", "12.5"],
@@ -53,7 +53,7 @@ fn frequency_casesensitive() {
     cmd.args(["--limit", "0"]).args(["--select", "h2"]);
 
     let mut got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
-    got.sort();
+    got.sort_unstable();
     let expected = vec![
         svec!["field", "value", "count", "percentage"],
         svec!["h2", "Y", "1", "14.28571"],
@@ -72,7 +72,7 @@ fn frequency_ignorecase() {
         .args(["--select", "h2"]);
 
     let mut got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
-    got.sort();
+    got.sort_unstable();
     let expected = vec![
         svec!["field", "value", "count", "percentage"],
         svec!["h2", "x", "1", "14.28571"],
@@ -82,6 +82,96 @@ fn frequency_ignorecase() {
     assert_eq!(got, expected);
 }
 
+#[test]
+fn frequency_trim() {
+    let wrk = Workdir::new("frequency_trim");
+
+    let rows = vec![
+        svec!["h1", "h2"],
+        svec!["a", "z"],
+        svec!["a", "y"],
+        svec!["a", "y"],
+        svec!["b", "z"],
+        svec!["a", "Y"],
+        svec!["", "z"],
+        svec!["(NULL)", "x"],
+        svec!["a ", " z"],
+        svec!["     A", "  Z   "],
+        svec!["  a  ", " Y "],
+        svec![" A     ", "y "],
+        svec!["a", "y "],
+        svec!["b", "y "],
+        svec!["b", "  Z   "],
+    ];
+
+    wrk.create("in.csv", rows);
+
+    let mut cmd = wrk.command("frequency");
+    cmd.arg("in.csv")
+        .args(["--limit", "0"])
+        .args(["--select", "h2"]);
+
+    wrk.assert_success(&mut cmd);
+
+    let mut got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    got.sort_unstable();
+    let expected = vec![
+        svec!["field", "value", "count", "percentage"],
+        svec!["h2", "Y", "2", "14.28571"],
+        svec!["h2", "Z", "2", "14.28571"],
+        svec!["h2", "x", "1", "7.14286"],
+        svec!["h2", "y", "5", "35.71429"],
+        svec!["h2", "z", "4", "28.57143"],
+    ];
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn frequency_no_trim() {
+    let wrk = Workdir::new("frequency_no_trim");
+
+    let rows = vec![
+        svec!["h1", "h2"],
+        svec!["a", "z"],
+        svec!["a", "y"],
+        svec!["a", "y"],
+        svec!["b", "z"],
+        svec!["a", "Y"],
+        svec!["", "z"],
+        svec!["(NULL)", "x"],
+        svec!["a ", " z"],
+        svec!["     A", "  Z   "],
+        svec!["  a  ", " Y "],
+        svec![" A     ", "y "],
+        svec!["a", "y "],
+        svec!["b", "y "],
+        svec!["b", "  Z   "],
+    ];
+
+    wrk.create("in.csv", rows);
+
+    let mut cmd = wrk.command("frequency");
+    cmd.arg("in.csv")
+        .args(["--limit", "0"])
+        .args(["--select", "h2"])
+        .arg("--no-trim");
+
+    let mut got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    got.sort_unstable();
+    let expected = vec![
+        svec!["field", "value", "count", "percentage"],
+        svec!["h2", "  Z   ", "2", "14.28571"],
+        svec!["h2", " Y ", "1", "7.14286"],
+        svec!["h2", " z", "1", "7.14286"],
+        svec!["h2", "Y", "1", "7.14286"],
+        svec!["h2", "x", "1", "7.14286"],
+        svec!["h2", "y", "2", "14.28571"],
+        svec!["h2", "y ", "3", "21.42857"],
+        svec!["h2", "z", "3", "21.42857"],
+    ];
+    assert_eq!(got, expected);
+}
+
 #[test]
 fn frequency_no_nulls() {
     let (wrk, mut cmd) = setup("frequency_no_nulls");
@@ -90,7 +180,7 @@ fn frequency_no_nulls() {
         .args(["--select", "h1"]);
 
     let mut got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
-    got.sort();
+    got.sort_unstable();
     let expected = vec![
         svec!["field", "value", "count", "percentage"],
         svec!["h1", "(NULL)", "1", "16.66667"],
@@ -106,7 +196,7 @@ fn frequency_nulls() {
     cmd.args(["--limit", "0"]).args(["--select", "h1"]);
 
     let mut got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
-    got.sort();
+    got.sort_unstable();
     let expected = vec![
         svec!["field", "value", "count", "percentage"],
         svec!["h1", "(NULL)", "1", "14.28571"],
@@ -123,7 +213,7 @@ fn frequency_limit() {
     cmd.args(["--limit", "1"]);
 
     let mut got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
-    got.sort();
+    got.sort_unstable();
     let expected = vec![
         svec!["field", "value", "count", "percentage"],
         svec!["h1", "Other (3)", "3", "42.85714"],
@@ -140,7 +230,7 @@ fn frequency_pct_dec_places() {
     cmd.args(["--limit", "1"]).args(["--pct-dec-places", "3"]);
 
     let mut got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
-    got.sort();
+    got.sort_unstable();
     let expected = vec![
         svec!["field", "value", "count", "percentage"],
         svec!["h1", "Other (3)", "3", "42.857"],
@@ -157,7 +247,7 @@ fn frequency_neg_pct_dec_places() {
     cmd.args(["--limit", "1"]).args(["--pct-dec-places", "-4"]);
 
     let mut got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
-    got.sort();
+    got.sort_unstable();
     let expected = vec![
         svec!["field", "value", "count", "percentage"],
         svec!["h1", "Other (3)", "3", "42.8571"],
@@ -174,7 +264,7 @@ fn frequency_limit_no_other() {
     cmd.args(["--limit", "1"]).args(["--other-text", "<NONE>"]);
 
     let mut got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
-    got.sort();
+    got.sort_unstable();
     let expected = vec![
         svec!["field", "value", "count", "percentage"],
         svec!["h1", "a", "4", "57.14286"],
@@ -189,7 +279,7 @@ fn frequency_negative_limit() {
     cmd.args(["--limit", "-4"]);
 
     let mut got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
-    got.sort();
+    got.sort_unstable();
     let expected = vec![
         svec!["field", "value", "count", "percentage"],
         svec!["h1", "Other (3)", "3", "42.85714"],
@@ -205,7 +295,7 @@ fn frequency_limit_threshold() {
     cmd.args(["--limit", "-4"]).args(["--lmt-threshold", "4"]);
 
     let mut got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
-    got.sort();
+    got.sort_unstable();
     let expected = vec![
         svec!["field", "value", "count", "percentage"],
         svec!["h1", "Other (3)", "3", "42.85714"],
@@ -221,7 +311,7 @@ fn frequency_limit_threshold_notmet() {
     cmd.args(["--limit", "-2"]).args(["--lmt-threshold", "3"]);
 
     let mut got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
-    got.sort();
+    got.sort_unstable();
     let expected = vec![
         svec!["field", "value", "count", "percentage"],
         svec!["h1", "(NULL)", "1", "14.28571"],
@@ -241,7 +331,7 @@ fn frequency_asc() {
     cmd.args(["--select", "h2"]).arg("--asc");
 
     let mut got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
-    got.sort();
+    got.sort_unstable();
     let expected = vec![
         svec!["field", "value", "count", "percentage"],
         svec!["h2", "Y", "1", "14.28571"],
@@ -260,7 +350,7 @@ fn frequency_asc_ignorecase() {
         .arg("--asc");
 
     let mut got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
-    got.sort();
+    got.sort_unstable();
     let expected = vec![
         svec!["field", "value", "count", "percentage"],
         svec!["h2", "x", "1", "14.28571"],
@@ -278,7 +368,7 @@ fn frequency_custom_other_text() {
         .args(["--other-text", "其他"]);
 
     let mut got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
-    got.sort();
+    got.sort_unstable();
     let expected = vec![
         svec!["field", "value", "count", "percentage"],
         svec!["h1", "a", "4", "57.14286"],
@@ -297,7 +387,7 @@ fn frequency_custom_other_text_sorted() {
         .arg("--other-sorted");
 
     let mut got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
-    got.sort();
+    got.sort_unstable();
     let expected = vec![
         svec!["field", "value", "count", "percentage"],
         svec!["h1", "Ibang halaga (3)", "3", "42.85714"],
@@ -315,7 +405,7 @@ fn frequency_other_sorted() {
         .arg("--other-sorted");
 
     let mut got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
-    got.sort();
+    got.sort_unstable();
     let expected = vec![
         svec!["field", "value", "count", "percentage"],
         svec!["h1", "Other (3)", "3", "42.85714"],
@@ -333,7 +423,7 @@ fn frequency_other_text_none() {
         .args(["--other-text", "<NONE>"]);
 
     let mut got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
-    got.sort();
+    got.sort_unstable();
     let expected = vec![
         svec!["field", "value", "count", "percentage"],
         svec!["h1", "a", "4", "57.14286"],
@@ -347,7 +437,7 @@ fn frequency_select() {
     cmd.args(["--limit", "0"]).args(["--select", "h2"]);
 
     let mut got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
-    got.sort();
+    got.sort_unstable();
     let expected = vec![
         svec!["field", "value", "count", "percentage"],
         svec!["h2", "Y", "1", "14.28571"],
@@ -358,6 +448,183 @@ fn frequency_select() {
     assert_eq!(got, expected);
 }
 
+#[test]
+fn frequency_all_unique() {
+    let wrk = Workdir::new("frequency_all_unique");
+    let testdata = wrk.load_test_file("boston311-100.csv");
+    let mut cmd = wrk.command("frequency");
+    cmd.args(["--select", "1"]).arg(testdata);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["field", "value", "count", "percentage"],
+        svec!["case_enquiry_id", "101004113298", "1", "1"],
+        svec!["case_enquiry_id", "101004113313", "1", "1"],
+        svec!["case_enquiry_id", "101004113348", "1", "1"],
+        svec!["case_enquiry_id", "101004113363", "1", "1"],
+        svec!["case_enquiry_id", "101004113371", "1", "1"],
+        svec!["case_enquiry_id", "101004113385", "1", "1"],
+        svec!["case_enquiry_id", "101004113386", "1", "1"],
+        svec!["case_enquiry_id", "101004113391", "1", "1"],
+        svec!["case_enquiry_id", "101004113394", "1", "1"],
+        svec!["case_enquiry_id", "101004113403", "1", "1"],
+        svec!["case_enquiry_id", "Other (90)", "90", "90"],
+    ];
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn frequency_all_unique_with_stats_cache() {
+    let wrk = Workdir::new("frequency_all_unique_with_stats_cache");
+    let testdata = wrk.load_test_file("boston311-100.csv");
+
+    let mut stats_cmd = wrk.command("stats");
+    stats_cmd
+        .arg(testdata.clone())
+        .arg("--cardinality")
+        .arg("--stats-binout");
+
+    wrk.assert_success(&mut stats_cmd);
+
+    let mut cmd = wrk.command("frequency");
+    cmd.args(["--select", "1"]).arg(testdata);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["field", "value", "count", "percentage"],
+        svec!["case_enquiry_id", "<ALL_UNIQUE>", "100", "100"],
+    ];
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn frequency_all_unique_with_stats_cache_alt_all_unique_text() {
+    let wrk = Workdir::new("frequency_all_unique_with_stats_cache_alt_all_unique_text");
+    let testdata = wrk.load_test_file("boston311-100.csv");
+
+    let mut stats_cmd = wrk.command("stats");
+    stats_cmd
+        .arg(testdata.clone())
+        .arg("--cardinality")
+        .arg("--stats-binout");
+
+    wrk.assert_success(&mut stats_cmd);
+
+    let mut cmd = wrk.command("frequency");
+    cmd.args(["--select", "1"])
+        // "<ALL_UNIQUE>" in German
+        .args(["--all-unique-text", "<ALLE EINZIGARTIG>"])
+        .arg(testdata);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["field", "value", "count", "percentage"],
+        svec!["case_enquiry_id", "<ALLE EINZIGARTIG>", "100", "100"],
+    ];
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn frequency_all_unique_force_stats_cache() {
+    let wrk = Workdir::new("frequency_all_unique_force_stats_cache");
+    let testdata = wrk.load_test_file("boston311-100.csv");
+
+    let mut cmd = wrk.command("frequency");
+    cmd.args(["--select", "1"])
+        .args(["--stats-mode", "force"])
+        .arg(testdata);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["field", "value", "count", "percentage"],
+        svec!["case_enquiry_id", "<ALL_UNIQUE>", "100", "100"],
+    ];
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn frequency_all_unique_stats_mode_none() {
+    let wrk = Workdir::new("frequency_all_unique_stats_mode_none");
+    let testdata = wrk.load_test_file("boston311-100.csv");
+
+    // create stats cache
+    let mut stats_cmd = wrk.command("stats");
+    stats_cmd
+        .arg(testdata.clone())
+        .arg("--cardinality")
+        .arg("--stats-binout");
+
+    wrk.assert_success(&mut stats_cmd);
+
+    // run frequency with stats-mode none, ignoring the stats cache
+    let mut cmd = wrk.command("frequency");
+    cmd.args(["--select", "1"])
+        .args(["--stats-mode", "none"])
+        .arg(testdata);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["field", "value", "count", "percentage"],
+        svec!["case_enquiry_id", "101004113298", "1", "1"],
+        svec!["case_enquiry_id", "101004113313", "1", "1"],
+        svec!["case_enquiry_id", "101004113348", "1", "1"],
+        svec!["case_enquiry_id", "101004113363", "1", "1"],
+        svec!["case_enquiry_id", "101004113371", "1", "1"],
+        svec!["case_enquiry_id", "101004113385", "1", "1"],
+        svec!["case_enquiry_id", "101004113386", "1", "1"],
+        svec!["case_enquiry_id", "101004113391", "1", "1"],
+        svec!["case_enquiry_id", "101004113394", "1", "1"],
+        svec!["case_enquiry_id", "101004113403", "1", "1"],
+        svec!["case_enquiry_id", "Other (90)", "90", "90"],
+    ];
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn frequency_issue1962() {
+    let wrk = Workdir::new("frequency_1962");
+    let testdata = wrk.load_test_file("data1962.csv");
+    let mut cmd = wrk.command("frequency");
+    cmd.args(["--limit", "15"]).arg(testdata.clone());
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["field", "value", "count", "percentage"],
+        svec!["year", "2024", "24", "8"],
+        svec!["year", "2023", "23", "7.66667"],
+        svec!["year", "2022", "22", "7.33333"],
+        svec!["year", "2021", "21", "7"],
+        svec!["year", "2020", "20", "6.66667"],
+        svec!["year", "2019", "19", "6.33333"],
+        svec!["year", "2018", "18", "6"],
+        svec!["year", "2017", "17", "5.66667"],
+        svec!["year", "2016", "16", "5.33333"],
+        svec!["year", "2015", "15", "5"],
+        svec!["year", "2014", "14", "4.66667"],
+        svec!["year", "2013", "13", "4.33333"],
+        svec!["year", "2012", "12", "4"],
+        svec!["year", "2011", "11", "3.66667"],
+        svec!["year", "2010", "10", "3.33333"],
+        svec!["year", "Other (9)", "45", "15"],
+    ];
+    assert_eq!(got, expected);
+
+    let mut cmd = wrk.command("frequency");
+    cmd.args(["--limit", "5"]).arg(testdata);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["field", "value", "count", "percentage"],
+        svec!["year", "2024", "24", "8"],
+        svec!["year", "2023", "23", "7.66667"],
+        svec!["year", "2022", "22", "7.33333"],
+        svec!["year", "2021", "21", "7"],
+        svec!["year", "2020", "20", "6.66667"],
+        svec!["year", "Other (19)", "190", "63.33333"],
+    ];
+    assert_eq!(got, expected);
+}
+
 // This tests that a frequency table computed by `qsv` is always the same
 // as the frequency table computed in memory.
 #[test]
@@ -367,7 +634,7 @@ fn prop_frequency() {
     }
     // Run on really small values because we are incredibly careless
     // with allocation.
-    qcheck_sized(p as fn(CsvData) -> bool, 2);
+    qcheck_sized(p as fn(CsvData) -> bool, 5);
 }
 
 // This tests that running the frequency command on a CSV file with these two
@@ -399,7 +666,7 @@ fn prop_frequency_indexed() {
     }
     // Run on really small values because we are incredibly careless
     // with allocation.
-    qcheck_sized(p as fn(CsvData) -> bool, 2);
+    qcheck_sized(p as fn(CsvData) -> bool, 5);
 }
 
 fn param_prop_frequency(name: &str, rows: CsvData, idx: bool) -> bool {
@@ -485,7 +752,7 @@ where
     T: ::std::hash::Hash + Ord + Clone,
 {
     let (mut freqs, _) = ftable.most_frequent();
-    freqs.sort();
+    freqs.sort_unstable();
     freqs
 }
 
