@@ -47,7 +47,7 @@ fn make_rows(left_only: bool, rows: Vec<Vec<String>>) -> Vec<Vec<String>> {
     if left_only {
         all_rows.push(svec!["city", "state"]);
     } else {
-        all_rows.push(svec!["city", "state", "place"]);
+        all_rows.push(svec!["city", "state", "city:places", "place"]);
     }
     all_rows.extend(rows.into_iter());
     all_rows
@@ -61,9 +61,9 @@ sqlp_test!(
         let expected = make_rows(
             false,
             vec![
-                svec!["Boston", "MA", "Logan Airport"],
-                svec!["Boston", "MA", "Boston Garden"],
-                svec!["Buffalo", "NY", "Ralph Wilson Stadium"],
+                svec!["Boston", "MA", "Boston", "Logan Airport"],
+                svec!["Boston", "MA", "Boston", "Boston Garden"],
+                svec!["Buffalo", "NY", "Buffalo", "Ralph Wilson Stadium"],
             ],
         );
         assert_eq!(got, expected);
@@ -78,11 +78,11 @@ sqlp_test!(
         let expected = make_rows(
             false,
             vec![
-                svec!["Boston", "MA", "Logan Airport"],
-                svec!["Boston", "MA", "Boston Garden"],
-                svec!["New York", "NY", ""],
-                svec!["San Francisco", "CA", ""],
-                svec!["Buffalo", "NY", "Ralph Wilson Stadium"],
+                svec!["Boston", "MA", "Boston", "Logan Airport"],
+                svec!["Boston", "MA", "Boston", "Boston Garden"],
+                svec!["New York", "NY", "", ""],
+                svec!["San Francisco", "CA", "", ""],
+                svec!["Buffalo", "NY", "Buffalo", "Ralph Wilson Stadium"],
             ],
         );
         assert_eq!(got, expected);
@@ -95,22 +95,22 @@ sqlp_test!(
         cmd.arg("select * from cities full outer join places on cities.city = places.city");
         let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
         let expected1 = vec![
-            svec!["city", "state", "city_right", "place"],
+            svec!["city", "state", "city:places", "place"],
             svec!["Boston", "MA", "Boston", "Logan Airport"],
             svec!["Boston", "MA", "Boston", "Boston Garden"],
             svec!["Buffalo", "NY", "Buffalo", "Ralph Wilson Stadium"],
             svec!["", "", "Orlando", "Disney World"],
-            svec!["San Francisco", "CA", "", ""],
             svec!["New York", "NY", "", ""],
+            svec!["San Francisco", "CA", "", ""],
         ];
         let expected2 = vec![
-            svec!["city", "state", "city_right", "place"],
+            svec!["city", "state", "city:places", "place"],
             svec!["Boston", "MA", "Boston", "Logan Airport"],
             svec!["Boston", "MA", "Boston", "Boston Garden"],
             svec!["Buffalo", "NY", "Buffalo", "Ralph Wilson Stadium"],
             svec!["", "", "Orlando", "Disney World"],
-            svec!["New York", "NY", "", ""],
             svec!["San Francisco", "CA", "", ""],
+            svec!["New York", "NY", "", ""],
         ];
         assert!(got == expected1 || got == expected2);
     }
@@ -144,7 +144,21 @@ fn sqlp_join_cross() {
 }
 
 #[test]
-// #[ignore = "temporarily disable due to a bug in polars aliasing"]
+fn sqlp_join_same_colname_1820() {
+    let wrk = Workdir::new("sqlp_join_same_colname_1820");
+    wrk.create("one.csv", vec![svec!["id", "data"], svec!["1", "open"]]);
+    wrk.create("two.csv", vec![svec!["id", "data"], svec!["1", "closed"]]);
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.args(["one.csv", "two.csv"])
+        .arg("SELECT _t_1.id, _t_2.data FROM _t_1 JOIN _t_2 ON _t_1.id = _t_2.id");
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![svec!["id", "data"], svec!["1", "closed"]];
+    assert_eq!(got, expected);
+}
+
+#[test]
 fn sqlp_boston311_groupby_orderby() {
     let wrk = Workdir::new("sqlp_boston311_groupby_orderby");
     let test_file = wrk.load_test_file("boston311-100.csv");
@@ -205,6 +219,65 @@ fn sqlp_boston311_groupby_orderby() {
     assert_eq!(got, expected);
 }
 
+#[test]
+fn sqlp_boston311_groupby_orderby_all() {
+    let wrk = Workdir::new("sqlp_boston311_groupby_orderby_all");
+    let test_file = wrk.load_test_file("boston311-100.csv");
+
+    let mut cmd = wrk.command("sqlp");
+
+    cmd.arg(&test_file)
+        .arg(r#"select ward, count(*) as cnt from "boston311-100" group by ward order by all"#);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["ward", "cnt"],
+        svec![" ", "1"],
+        svec!["01", "1"],
+        svec!["02", "1"],
+        svec!["03", "2"],
+        svec!["04", "1"],
+        svec!["06", "1"],
+        svec!["07", "1"],
+        svec!["1", "1"],
+        svec!["10", "1"],
+        svec!["14", "4"],
+        svec!["16", "1"],
+        svec!["17", "2"],
+        svec!["18", "1"],
+        svec!["19", "1"],
+        svec!["21", "1"],
+        svec!["22", "2"],
+        svec!["3", "5"],
+        svec!["7", "1"],
+        svec!["8", "1"],
+        svec!["9", "1"],
+        svec!["Ward 1", "6"],
+        svec!["Ward 10", "1"],
+        svec!["Ward 11", "2"],
+        svec!["Ward 12", "1"],
+        svec!["Ward 13", "4"],
+        svec!["Ward 14", "1"],
+        svec!["Ward 15", "1"],
+        svec!["Ward 16", "4"],
+        svec!["Ward 17", "1"],
+        svec!["Ward 18", "3"],
+        svec!["Ward 19", "3"],
+        svec!["Ward 2", "1"],
+        svec!["Ward 20", "5"],
+        svec!["Ward 21", "2"],
+        svec!["Ward 22", "1"],
+        svec!["Ward 3", "10"],
+        svec!["Ward 4", "5"],
+        svec!["Ward 5", "5"],
+        svec!["Ward 6", "7"],
+        svec!["Ward 7", "3"],
+        svec!["Ward 8", "3"],
+        svec!["Ward 9", "1"],
+    ];
+    assert_eq!(got, expected);
+}
+
 #[test]
 // #[ignore = "temporarily disable due to a bug in polars aliasing"]
 fn sqlp_boston311_groupby_orderby_with_table_alias() {
@@ -284,7 +357,6 @@ fn sqlp_boston311_wnull_value() {
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
     let expected = vec![
         svec!["location_street_name", "location_zipcode"],
-        svec!["Not Specified", "Not Specified"],
         svec!["INTERSECTION Asticou Rd & Washington St", "Not Specified"],
         svec![
             "INTERSECTION Charles River Plz & Cambridge St",
@@ -292,6 +364,10 @@ fn sqlp_boston311_wnull_value() {
         ],
         svec!["INTERSECTION Columbia Rd & E Cottage St", "Not Specified"],
         svec!["INTERSECTION E Canton St & Albany St", "Not Specified"],
+        svec![
+            "INTERSECTION Gallivan Blvd & Washington St",
+            "Not Specified"
+        ],
     ];
     assert_eq!(got, expected);
 }
@@ -825,6 +901,9 @@ fn sqlp_compress() {
     assert_eq!(got, expected);
 }
 
+// disable this test on windows as it fails as the expected output is different
+// due to the different line endings
+#[cfg(not(target_os = "windows"))]
 #[test]
 fn sqlp_boston311_explain() {
     let wrk = Workdir::new("sqlp_boston311_explain");
@@ -841,13 +920,12 @@ fn sqlp_boston311_explain() {
 "SORT BY [col(""avg_tat""), col(""ward"")]"
   AGGREGATE
 "  	[[(col(""closed_dt"")) - (col(""open_dt""))].mean().strict_cast(Float64).alias(""avg_tat"")] BY [col(""ward"")] FROM"
-""
-      Csv SCAN"#;
+    Csv SCAN"#;
     assert!(got.starts_with(expected_begin));
 
-    let expected_end = r#"boston311-100.csv
-      PROJECT 4/29 COLUMNS
-"      SELECTION: [(col(""case_status"")) == (String(Closed))]""#;
+    let expected_end = r#"boston311-100.csv]
+    PROJECT 4/29 COLUMNS
+"    SELECTION: [(col(""case_status"")) == (String(Closed))]""#;
     assert!(got.ends_with(expected_end));
 }
 
@@ -1037,6 +1115,48 @@ fn sqlp_boston311_case_expression() {
     assert_eq!(got, expected);
 }
 
+#[test]
+fn sqlp_boston311_case_expression_streaming() {
+    let wrk = Workdir::new("sqlp_boston311_case_expression_streaming");
+    let test_file = wrk.load_test_file("boston311-100.csv");
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.arg(&test_file)
+        .arg(
+            r#"SELECT case_enquiry_id, 
+           CASE closed_dt is null and case_title ~* 'graffiti' 
+              WHEN True THEN 'Yes' 
+              WHEN False THEN 'No' 
+              ELSE 'N/A'
+           END as graffiti_related
+           from _t_1
+           where case_status = 'Open'"#,
+        )
+        .arg("--streaming");
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["case_enquiry_id", "graffiti_related"],
+        svec!["101004143000", "No"],
+        svec!["101004155594", "No"],
+        svec!["101004154423", "No"],
+        svec!["101004141848", "No"],
+        svec!["101004113313", "No"],
+        svec!["101004113751", "Yes"],
+        svec!["101004113902", "Yes"],
+        svec!["101004113473", "No"],
+        svec!["101004113604", "No"],
+        svec!["101004114154", "Yes"],
+        svec!["101004114383", "No"],
+        svec!["101004114795", "Yes"],
+        svec!["101004118346", "Yes"],
+        svec!["101004115302", "No"],
+        svec!["101004115066", "No"],
+    ];
+
+    assert_eq!(got, expected);
+}
+
 #[test]
 fn sqlp_boston311_case() {
     let wrk = Workdir::new("sqlp_boston311_case");
@@ -1166,9 +1286,9 @@ fn sqlp_sql_join_on_subquery() {
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
     let expected = vec![
-        svec!["idx", "val", "val_right"],
-        svec!["3", "A0C", "A0C"],
-        svec!["4", "a0c", "a0c"],
+        svec!["idx", "val", "idx:t2", "val:t2"],
+        svec!["3", "A0C", "3", "A0C"],
+        svec!["4", "a0c", "4", "a0c"],
     ];
 
     assert_eq!(got, expected);
@@ -1209,9 +1329,9 @@ fn sqlp_sql_from_subquery() {
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
     let expected = vec![
-        svec!["idx", "val", "val_right"],
-        svec!["3", "A0C", "A0C"],
-        svec!["4", "a0c", "a0c"],
+        svec!["idx", "val", "idx:t2", "val:t2"],
+        svec!["3", "A0C", "3", "A0C"],
+        svec!["4", "a0c", "4", "a0c"],
     ];
 
     assert_eq!(got, expected);
@@ -1248,6 +1368,127 @@ fn sqlp_sql_tsv() {
     assert_eq!(got, expected);
 }
 
+#[test]
+fn sqlp_sql_ssv() {
+    let wrk = Workdir::new("sqlp_sql_ssv");
+    wrk.create(
+        "test.csv",
+        vec![
+            svec!["idx", "val"],
+            svec!["0", "ABC"],
+            svec!["1", "abc"],
+            svec!["2", "000"],
+            svec!["3", "A0C"],
+            svec!["4", "a0c"],
+        ],
+    );
+
+    let output_file = wrk.path("output.ssv").to_string_lossy().to_string();
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.arg("test.csv")
+        .arg("SELECT * FROM test")
+        .args(["--output", &output_file]);
+
+    wrk.assert_success(&mut cmd);
+
+    let got = wrk.read_to_string(&output_file);
+
+    let expected = "idx;val\n0;ABC\n1;abc\n2;000\n3;A0C\n4;a0c\n";
+
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn sqlp_sql_ssv_output() {
+    let wrk = Workdir::new("sqlp_sql_ssv_output");
+    wrk.create_with_delim(
+        "test.ssv",
+        vec![
+            svec!["idx", "val"],
+            svec!["0", "ABC"],
+            svec!["1", "abc"],
+            svec!["2", "000"],
+            svec!["3", "A0C"],
+            svec!["4", "a0c"],
+        ],
+        b';',
+    );
+
+    let output_file = wrk.path("output.ssv").to_string_lossy().to_string();
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.arg("test.ssv")
+        .arg("SELECT * FROM test")
+        .args(["--output", &output_file]);
+
+    wrk.assert_success(&mut cmd);
+
+    let got = wrk.read_to_string(&output_file);
+
+    let expected = "idx;val\n0;ABC\n1;abc\n2;000\n3;A0C\n4;a0c\n";
+
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn sqlp_issue2014() {
+    let wrk = Workdir::new("sqlp_issue2014");
+    wrk.create_with_delim(
+        "test.ssv",
+        vec![
+            svec!["id", "item", "price"],
+            svec!["0", "wallet", "9.99"],
+            svec!["1", "comb", "1.39"],
+            svec!["2", "pencil", "0.49"],
+        ],
+        b';',
+    );
+
+    let output_file = wrk.path("output.ssv.sz").to_string_lossy().to_string();
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.arg("test.ssv")
+        .arg("SELECT * FROM test")
+        .args(["--output", &output_file]);
+
+    wrk.assert_success(&mut cmd);
+
+    let mut cmd2 = wrk.command("slice"); // DevSkim: ignore DS126858
+    cmd2.arg(output_file.clone()); // DevSkim: ignore DS126858
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd2); // DevSkim: ignore DS126858
+    let expected = vec![
+        svec!["id", "item", "price"],
+        svec!["0", "wallet", "9.99"],
+        svec!["1", "comb", "1.39"],
+        svec!["2", "pencil", "0.49"],
+    ];
+
+    assert_eq!(got, expected);
+
+    let mut cmd2 = wrk.command("snappy"); // DevSkim: ignore DS126858
+    cmd2.arg("decompress").arg(output_file.clone()); // DevSkim: ignore DS126858
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd2); // DevSkim: ignore DS126858
+    let expected = vec![
+        ["id;item;price"],
+        ["0;wallet;9.99"],
+        ["1;comb;1.39"],
+        ["2;pencil;0.49"],
+    ];
+
+    assert_eq!(got, expected);
+
+    let mut cmd2 = wrk.command("headers"); // DevSkim: ignore DS126858
+    cmd2.arg(output_file); // DevSkim: ignore DS126858
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd2); // DevSkim: ignore DS126858
+    let expected = vec![["1   id"], ["2   item"], ["3   price"]];
+
+    assert_eq!(got, expected);
+}
+
 #[test]
 fn sqlp_binary_functions() {
     let wrk = Workdir::new("sqlp_sql_binary_functions");
@@ -1483,7 +1724,11 @@ fn sqlp_compound_join_basic() {
     wrk.assert_success(&mut cmd);
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
-    let expected = vec![svec!["a", "b",], svec!["2", "3"], svec!["3", "4"]];
+    let expected = vec![
+        svec!["a", "b", "a:test2", "b:test2"],
+        svec!["2", "3", "2", "3"],
+        svec!["3", "4", "3", "4"],
+    ];
 
     assert_eq!(got, expected);
 }
@@ -1529,9 +1774,9 @@ fn sqlp_compound_join_diff_colnames() {
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
     let expected = vec![
-        svec!["a", "b", "c"],
-        svec!["2", "2", "8"],
-        svec!["3", "3", "9"],
+        svec!["a", "b", "a:test2", "b:test2", "c"],
+        svec!["2", "2", "2", "2", "8"],
+        svec!["3", "3", "3", "3", "9"],
     ];
 
     assert_eq!(got, expected);
@@ -1593,9 +1838,9 @@ fn sqlp_compound_join_three_tables() {
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
     let expected = vec![
-        svec!["a", "b", "c"],
-        svec!["2", "3", "3"],
-        svec!["3", "4", "4"],
+        svec!["a", "b", "a:test2", "b:test2", "a:test3", "b:test3", "c"],
+        svec!["2", "3", "2", "3", "2", "3", "3"],
+        svec!["3", "4", "3", "4", "3", "4", "4"],
     ];
 
     assert_eq!(got, expected);
@@ -1723,6 +1968,42 @@ fn sqlp_modulo() {
     assert_eq!(got, expected);
 }
 
+#[test]
+fn sqlp_try_cast() {
+    let wrk = Workdir::new("sqlp_try_cast");
+    wrk.create(
+        "test.csv",
+        vec![
+            svec!["foo", "bar"],
+            svec!["65432", "1999-12-31"],
+            svec!["101010", "N/A"],
+            svec!["-3333", "2024-01-01"],
+        ],
+    );
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.arg("test.csv").arg(
+        r#"
+        SELECT
+            try_cast(foo as uint2),
+            try_cast(bar as DATE)
+        FROM test
+"#,
+    );
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["foo", "bar"],
+        svec!["65432", "1999-12-31"],
+        svec!["", ""],
+        svec!["", "2024-01-01"],
+    ];
+
+    assert_eq!(got, expected);
+}
+
 #[test]
 fn sqlp_stddev_variance() {
     let wrk = Workdir::new("sqlp_stddev_variance");
@@ -1829,23 +2110,23 @@ fn sqlp_date_part() {
         .arg(
             r#"
         SELECT 
-            DATE_PART(datecol, 'isoyear') as c1,
-            DATE_PART(datecol, 'month') as c2,
-            DATE_PART(datecol, 'day') as c3,
-            DATE_PART(datecol, 'hour') as c4,
-            DATE_PART(datecol, 'minute') as c5,
-            DATE_PART(datecol, 'second') as c6,
-            DATE_PART(datecol, 'millisecond') as c61,
-            DATE_PART(datecol, 'microsecond') as c62,
-            DATE_PART(datecol, 'nanosecond') as c63,
-            DATE_PART(datecol, 'isoweek') as c7,
-            DATE_PART(datecol, 'dayofyear') as c8,
-            DATE_PART(datecol, 'dayofweek') as c9,
-            DATE_PART(datecol, 'time') as c10,
-            DATE_PART(datecol, 'decade') as c11,
-            DATE_PART(datecol, 'century') as c12,
-            DATE_PART(datecol, 'millennium') as c13,
-            DATE_PART(datecol, 'quarter') as c14,
+            DATE_PART('isoyear', datecol) as c1,
+            DATE_PART('month', datecol) as c2,
+            DATE_PART('day', datecol) as c3,
+            DATE_PART('hour', datecol) as c4,
+            DATE_PART('minute', datecol) as c5,
+            DATE_PART('second', datecol) as c6,
+            DATE_PART('millisecond', datecol) as c61,
+            DATE_PART('microsecond', datecol) as c62,
+            DATE_PART('nanosecond', datecol) as c63,
+            DATE_PART('isoweek', datecol) as c7,
+            DATE_PART('dayofyear', datecol) as c8,
+            DATE_PART('dayofweek', datecol) as c9,
+            DATE_PART('time', datecol) as c10,
+            DATE_PART('decade', datecol) as c11,
+            DATE_PART('century', datecol) as c12,
+            DATE_PART('millennium', datecol) as c13,
+            DATE_PART('quarter', datecol) as c14,
       FROM datestbl
 "#,
         )
@@ -1884,24 +2165,550 @@ fn sqlp_date_part() {
 }
 
 #[test]
-fn sqlp_date() {
-    let wrk = Workdir::new("sqlp_date");
-    wrk.create("dummy.csv", vec![svec!["c1"], svec![""]]);
+fn sqlp_date_part_tz() {
+    let wrk = Workdir::new("sqlp_date_part_tz");
+    wrk.create(
+        "datestbl.csv",
+        vec![
+            svec!["datecol"],
+            svec!["2012-01-20T10:30:20-05:00"],
+            svec!["2012-11-05T10:30:20.1234Z"],
+            svec!["1999-03-15T15:30:20.1234+01:00"],
+            svec!["1978-08-05T23:30:20.1234-01:00"],
+        ],
+    );
 
     let mut cmd = wrk.command("sqlp");
-    cmd.arg("dummy.csv").arg(
+    cmd.arg("datestbl.csv")
+        .arg(
+            r#"
+        SELECT
+            DATE_PART('isoyear', datecol) as c1,
+            DATE_PART('month', datecol) as c2,
+            DATE_PART('day', datecol) as c3,
+            DATE_PART('hour', datecol) as c4,
+            DATE_PART('minute', datecol) as c5,
+            DATE_PART('second', datecol) as c6,
+            DATE_PART('millisecond', datecol) as c61,
+            DATE_PART('microsecond', datecol) as c62,
+            DATE_PART('nanosecond', datecol) as c63,
+            DATE_PART('isoweek', datecol) as c7,
+            DATE_PART('dayofyear', datecol) as c8,
+            DATE_PART('dayofweek', datecol) as c9,
+            DATE_PART('time', datecol) as c10,
+            DATE_PART('decade', datecol) as c11,
+            DATE_PART('century', datecol) as c12,
+            DATE_PART('millennium', datecol) as c13,
+            DATE_PART('quarter', datecol) as c14,
+            DATE_PART('timezone', datecol) as c15,
+            DATE_PART('epoch', datecol) as c16,
+      FROM datestbl
+"#,
+        )
+        .arg("--try-parsedates");
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec![
+            "c1", "c2", "c3", "c4", "c5", "c6", "c61", "c62", "c63", "c7", "c8", "c9", "c10",
+            "c11", "c12", "c13", "c14", "c15", "c16"
+        ],
+        svec![
+            "2012",
+            "1",
+            "20",
+            "15",
+            "30",
+            "20",
+            "20000.0",
+            "20000000.0",
+            "20000000000.0",
+            "3",
+            "20",
+            "5",
+            "15:30:20.000000000",
+            "201",
+            "21",
+            "3",
+            "1",
+            "0",
+            "1327073420.0"
+        ],
+        svec![
+            "2012",
+            "11",
+            "5",
+            "10",
+            "30",
+            "20",
+            "20123.4",
+            "20123400.0",
+            "20123400000.0",
+            "45",
+            "310",
+            "1",
+            "10:30:20.123400000",
+            "201",
+            "21",
+            "3",
+            "4",
+            "0",
+            "1352111420.1234"
+        ],
+        svec![
+            "1999",
+            "3",
+            "15",
+            "14",
+            "30",
+            "20",
+            "20123.4",
+            "20123400.0",
+            "20123400000.0",
+            "11",
+            "74",
+            "1",
+            "14:30:20.123400000",
+            "199",
+            "20",
+            "2",
+            "1",
+            "0",
+            "921508220.1234"
+        ],
+        svec![
+            "1978",
+            "8",
+            "6",
+            "0",
+            "30",
+            "20",
+            "20123.4",
+            "20123400.0",
+            "20123400000.0",
+            "31",
+            "218",
+            "0",
+            "00:30:20.123400000",
+            "197",
+            "20",
+            "2",
+            "3",
+            "0",
+            "271211420.1234"
+        ],
+    ];
+
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn sqlp_date() {
+    let wrk = Workdir::new("sqlp_date");
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.arg("SKIP_INPUT").arg(
         r#"
         SELECT 
             DATE('2021-03-15') as c1,
             DATE('2021-03-15 10:30:20', '%Y-%m-%d %H:%M:%S') as c2,
-        FROM dummy
-"#,
+            DATE('03-15-2021 10:30:20 AM EST', '%m-%d-%Y %I:%M:%S %p %Z') as c3"#,
     );
 
     wrk.assert_success(&mut cmd);
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
-    let expected = vec![svec!["c1", "c2"], svec!["2021-03-15", "2021-03-15"]];
+    // this is the documented behavior of the date function
+    // use STRFTIME and STRPTIME for more control
+    let expected = vec![
+        svec!["c1", "c2", "c3"],
+        svec!["2021-03-15", "2021-03-15", "2021-03-15"],
+    ];
+
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn sqlp_date_strftime() {
+    let wrk = Workdir::new("sqlp_date_strftime");
+
+    wrk.create(
+        "data.csv",
+        vec![
+            svec!["dtm", "dt", "tm"],
+            svec!["1972-03-06 23:50:03", "1978-07-05", "10:10:10"],
+            svec!["1980-09-30 01:25:50", "1969-12-31", "22:33:55"],
+        ],
+    );
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.arg("data.csv")
+        .arg(
+            r#"
+        SELECT
+      STRFTIME(dtm,'%m.%d.%Y/%T') AS s_dtm,
+      STRFTIME(dt,'%B %d, %Y') AS s_dt,
+      STRFTIME(tm,'%S.%M.%H') AS s_tm,
+    FROM data"#,
+        )
+        .arg("--try-parsedates");
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["s_dtm", "s_dt", "s_tm"],
+        svec!["03.06.1972/23:50:03", "July 05, 1978", "10.10.10"],
+        svec!["09.30.1980/01:25:50", "December 31, 1969", "55.33.22"],
+    ];
+
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn sqlp_read_jsonl() {
+    let wrk = Workdir::new("sqlp_read_jsonl");
+
+    let test_jsonl = wrk.load_test_file("boston311-10.jsonl");
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.arg("SKIP_INPUT").arg(
+        format!(
+            "SELECT * FROM read_json('{}') WHERE source = 'City Worker App'",
+            test_jsonl
+        )
+        .as_str(),
+    );
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec![
+            "case_enquiry_id",
+            "open_dt",
+            "target_dt",
+            "closed_dt",
+            "ontime",
+            "case_status",
+            "closure_reason",
+            "case_title",
+            "subject",
+            "reason",
+            "type",
+            "queue",
+            "department",
+            "submittedphoto",
+            "closedphoto",
+            "location",
+            "fire_district",
+            "pwd_district",
+            "city_council_district",
+            "police_district",
+            "neighborhood",
+            "neighborhood_services_district",
+            "ward",
+            "precinct",
+            "location_street_name",
+            "location_zipcode",
+            "latitude",
+            "longitude",
+            "source"
+        ],
+        svec![
+            "101004120108",
+            "2022-01-08 12:54:49",
+            "2022-01-11 08:30:00",
+            "2022-01-09 06:43:06",
+            "ONTIME",
+            "Closed",
+            "Case Closed. Closed date : Sun Jan 09 06:43:06 EST 2022 Noted ",
+            "CE Collection",
+            "Public Works Department",
+            "Street Cleaning",
+            "CE Collection",
+            "PWDx_District 1C: Downtown",
+            "PWDx",
+            "",
+            "",
+            "198 W Springfield St  Roxbury  MA  02118",
+            "4",
+            "1C",
+            "7",
+            "D4",
+            "South End",
+            "6",
+            "Ward 9",
+            "0902",
+            "198 W Springfield St",
+            "2118",
+            "42.3401",
+            "-71.0803",
+            "City Worker App"
+        ],
+        svec![
+            "101004141354",
+            "2022-01-20 08:07:49",
+            "2022-01-21 08:30:00",
+            "2022-01-20 08:45:03",
+            "ONTIME",
+            "Closed",
+            "Case Closed. Closed date : Thu Jan 20 08:45:03 EST 2022 Noted ",
+            "CE Collection",
+            "Public Works Department",
+            "Street Cleaning",
+            "CE Collection",
+            "PWDx_District 1B: North End",
+            "PWDx",
+            "",
+            "",
+            "21-23 Temple St  Boston  MA  02114",
+            "3",
+            "1B",
+            "1",
+            "A1",
+            "Beacon Hill",
+            "3",
+            "Ward 3",
+            "0306",
+            "21-23 Temple St",
+            "2114",
+            "42.3606",
+            "-71.0638",
+            "City Worker App"
+        ],
+        svec![
+            "101004141367",
+            "2022-01-20 08:15:45",
+            "2022-01-21 08:30:00",
+            "2022-01-20 08:45:12",
+            "ONTIME",
+            "Closed",
+            "Case Closed. Closed date : Thu Jan 20 08:45:12 EST 2022 Noted ",
+            "CE Collection",
+            "Public Works Department",
+            "Street Cleaning",
+            "CE Collection",
+            "PWDx_District 1B: North End",
+            "PWDx",
+            "",
+            "",
+            "12 Derne St  Boston  MA  02114",
+            "3",
+            "1B",
+            "1",
+            "A1",
+            "Beacon Hill",
+            "3",
+            "Ward 3",
+            "0306",
+            "12 Derne St",
+            "2114",
+            "42.3596",
+            "-71.0634",
+            "City Worker App"
+        ],
+    ];
+
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn sqlp_string_like_ops() {
+    let wrk = Workdir::new("sqlp_string_like_ops");
+
+    wrk.create(
+        "likedata.csv",
+        vec![
+            svec!["x", "y"],
+            svec!["aaa", "abc"],
+            svec!["bbb", "b"],
+            svec!["a", "aa"],
+        ],
+    );
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.arg("likedata.csv").arg(
+        r#"
+        SELECT
+            x,
+            x ^@ 'a' AS x_starts_with_a,
+            x ~~* '%B' AS x_ends_with_b_case_insensitive,
+            x ^@ y AS x_starts_with_y,
+            x ~~ '%a' AS x_ends_with_a
+        FROM likedata"#,
+    );
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec![
+            "x",
+            "x_starts_with_a",
+            "x_ends_with_b_case_insensitive",
+            "x_starts_with_y",
+            "x_ends_with_a"
+        ],
+        svec!["aaa", "true", "false", "false", "true"],
+        svec!["bbb", "false", "true", "true", "false"],
+        svec!["a", "true", "false", "false", "true"],
+    ];
+
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn sqlp_natural_join() {
+    let wrk = Workdir::new("sqlp_natural_join");
+
+    wrk.create(
+        "data1.csv",
+        vec![
+            svec!["CharacterID", "FirstName", "LastName"],
+            svec!["1", "Jerna Morat", "Gurgeh"],
+            svec!["2", "Cheradenine", "Zakalwe"],
+            svec!["3", "Byr", "Genar-Hofoen"],
+        ],
+    );
+
+    wrk.create(
+        "data2.csv",
+        vec![
+            svec!["CharacterID", "Book"],
+            svec!["1", "Player of Games"],
+            svec!["2", "Use of Weapons"],
+            svec!["3", "Excession"],
+        ],
+    );
+
+    wrk.create(
+        "data3.csv",
+        vec![
+            svec!["CharacterID", "Ship"],
+            svec!["1", "Limiting Factor"],
+            svec!["2", "Xenophobe"],
+            svec!["3", "Grey Area"],
+        ],
+    );
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.args(["data1.csv", "data2.csv", "data3.csv"]).arg(
+        r#"SELECT COLUMNS('^[^:]+$')
+  FROM data1
+    NATURAL JOIN data2
+    NATURAL JOIN data3
+  ORDER BY CharacterID"#,
+    );
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["CharacterID", "FirstName", "LastName", "Book", "Ship"],
+        svec![
+            "1",
+            "Jerna Morat",
+            "Gurgeh",
+            "Player of Games",
+            "Limiting Factor"
+        ],
+        svec!["2", "Cheradenine", "Zakalwe", "Use of Weapons", "Xenophobe"],
+        svec!["3", "Byr", "Genar-Hofoen", "Excession", "Grey Area"],
+    ];
+
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn sqlp_star_ilike() {
+    let wrk = Workdir::new("sqlp_star_ilike");
+
+    wrk.create(
+        "starlikedata.csv",
+        vec![
+            svec!["ID", "FirstName", "LastName", "Address", "City"],
+            svec!["333", "Bruce", "Wayne", "The Batcave", "Gotham"],
+            svec!["666", "Diana", "Prince", "Paradise Island", "Themyscira"],
+            svec!["999", "Clark", "Kent", "Fortress of Solitude", "Metropolis"],
+        ],
+    );
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.arg("starlikedata.csv").arg(
+        r#"
+        SELECT * ILIKE '%a%e%'
+  FROM starlikedata
+  ORDER BY FirstName"#,
+    );
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["FirstName", "LastName", "Address"],
+        svec!["Bruce", "Wayne", "The Batcave"],
+        svec!["Clark", "Kent", "Fortress of Solitude"],
+        svec!["Diana", "Prince", "Paradise Island"],
+    ];
+
+    assert_eq!(got, expected);
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.arg("starlikedata.csv").arg(
+        r#"
+        SELECT * ILIKE '%I%' RENAME (FirstName AS Name) 
+  FROM starlikedata
+  ORDER BY 3 DESC"#,
+    );
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["ID", "Name", "City"],
+        svec!["666", "Diana", "Themyscira"],
+        svec!["999", "Clark", "Metropolis"],
+        svec!["333", "Bruce", "Gotham"],
+    ];
+
+    assert_eq!(got, expected);
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.arg("starlikedata.csv").arg(
+        r#"
+        SELECT * EXCLUDE (ID, City, LastName) RENAME FirstName AS Name
+  FROM starlikedata
+  ORDER BY Name"#,
+    );
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["Name", "Address"],
+        svec!["Bruce", "The Batcave"],
+        svec!["Clark", "Fortress of Solitude"],
+        svec!["Diana", "Paradise Island"],
+    ];
+
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn sqlp_skip_input() {
+    let wrk = Workdir::new("sqlp_skip_input");
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.arg("SKIP_INPUT")
+        .arg("SELECT 1 AS one, '2' AS two, 3.0 AS three");
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![svec!["one", "two", "three"], svec!["1", "2", "3.0"]];
 
     assert_eq!(got, expected);
 }
